import re
from abc import ABC, abstractmethod
from typing import Dict, List, Optional

import spacy
from nltk import SnowballStemmer


class BasePreprocessor(ABC):
    @abstractmethod
    def __call__(self, text: str) -> str:
        raise NotImplementedError


class WhitespaceRemovePreprocessor(BasePreprocessor):
    def __call__(self, text: str) -> str:
        """
        Basic cleaning :
            1. remove any hyphen followed by one or more whitespace
            2. removes double white spaces between words
            3. removes beginning and trailing whitespace in a string
        :param text: input string
        :return: str
        """
        text = re.sub(r"-\s+", "", text)
        text = re.sub(r"\s+", " ", text)
        text = re.sub(r"\s$|^\s", "", text)
        return text


class MakeLowerCasePreprocessor(BasePreprocessor):
    def __call__(self, text: str) -> str:
        """
        Makes all string lower case
        """
        return text.lower()


class StemPreprocessor(BasePreprocessor):
    def __init__(self, language: str = "english"):
        self.stemmer = SnowballStemmer(language)

    def __call__(self, text: str) -> str:
        """
        Stemming operation using nltk
        """
        return " ".join([self.stemmer.stem(word) for word in text.split()])


class LemmaPreprocessor(BasePreprocessor):
    def __init__(self, model_name: str = "en_core_web_sm"):
        self.spacy_model = spacy.load(model_name)

    def __call__(self, text: str) -> str:
        """
        Lemmatization operation using nltk
        """
        return " ".join([token.lemma_ for token in self.spacy_model(text)])


class ModelSpecificPreprocessor(BasePreprocessor):
    def __call__(self, text: str) -> str:
        """
        Basic cleaning :
            1. replaces mentions with generic placeholder - @user
            2. replaces urls with generic placeholder - http
        :param text: input string
        :return: str
        """
        new_text = []
        for t in text.split(" "):
            t = "@user" if t.startswith("@") and len(t) > 1 else t
            t = "http" if t.startswith("http") else t
            new_text.append(t)
        return " ".join(new_text)


class StackedPreprocessor(BasePreprocessor):
    def __init__(self, preprocessors: Optional[Dict[str, Optional[Dict]]] = None):
        if preprocessors is None:
            preprocessors = {}

        self.preprocessors = [PREPROCESSORS[name](**params) for name, params in preprocessors.items()]

    def __call__(self, text_batch: List[str]) -> List[str]:
        processed_batch = []
        for text in text_batch:
            for preprocessor in self.preprocessors:
                text = preprocessor(text)
            processed_batch.append(text)
        return processed_batch


PREPROCESSORS = {
    "base_cleaning": WhitespaceRemovePreprocessor,
    "lowcase": MakeLowerCasePreprocessor,
    "stem": StemPreprocessor,
    "lemma": LemmaPreprocessor,
    "cardiffnlp_processor": ModelSpecificPreprocessor,
}



import requests
import torch
import yaml
from flask import Flask, jsonify, request
from sa_app.common.utils import parse_args
from sa_app.inference.inference import InferenceEngine

app = Flask(__name__)
args = parse_args()
config = yaml.safe_load(open(args.config, "r"))
device_in_use = "cuda" if torch.cuda.is_available() else "cpu"
ie_obj = InferenceEngine(
    inference_params=config["inference_params"],
    training_params=config["training_params"],
    dataset_params=config["dataset_params"],
    device=device_in_use,
)


@app.route("/sentiment_analysis", methods=["POST"])
def get_sentiment():
    new_tweet = {"id": request.json["id"], "tweet_content": request.json["tweet_content"]}
    result_data = {"result": ie_obj.perform_inference(new_tweet["tweet_content"])}
    db_update_status = requests.post(
        "http://mongo-writer-service:5002/update_tweet_sentiment",
        json={"u_id": new_tweet["id"], "sentiment_value": result_data["result"]},
    )
    result_data["db_update_status"] = db_update_status.status_code
    return jsonify(result_data)


@app.route("/home")
def test_fn():
    return "Hello World!!!"


if __name__ == "__main__":
    app.run(host="0.0.0.0", port=5000)



import torch
import torch.nn as nn
from transformers import AutoModelForSequenceClassification

MODEL = {"fine_tune_base_model": AutoModelForSequenceClassification}


class CustomClassificationHead(nn.Module):
    def __init__(self, input_dim: int = 768, num_labels: int = 3):
        super(CustomClassificationHead, self).__init__()
        self.dense = nn.Linear(input_dim, input_dim)
        self.dropout = nn.Dropout(p=0.1)
        self.out_proj = nn.Linear(input_dim, num_labels)

    def forward(self, features):
        x = self.dense(features)
        x = torch.relu(x)
        x = self.dropout(x)
        x = self.out_proj(x)
        return x


class Model:
    def __init__(self):
        super().__init__()

    @classmethod
    def from_config(cls, mode: str):
        try:
            class_ = MODEL[mode]
        except KeyError:
            raise KeyError(f'Mode type "{mode}" is not implemented.')

        return class_



import os
from typing import Dict, List, Tuple

import pytorch_lightning as pl
import torch
import wandb
import yaml
from pytorch_lightning.callbacks import LearningRateMonitor, ModelCheckpoint
from pytorch_lightning.loggers import WandbLogger
from sa_app.common.utils import init_model_loggers, parse_args
from sa_app.data.data import InitializeDataset, SentimentIterableDataset
from sa_app.models.model import CustomClassificationHead, Model
from sa_app.training.lightning_model_wrapper import LightningModelWrapper
from transformers import AutoConfig, AutoModelForSequenceClassification, AutoTokenizer


def init_model_callbacks(
    training_params: dict,
) -> List[LearningRateMonitor | ModelCheckpoint]:
    model_checkpoint = ModelCheckpoint(
        dirpath=training_params["callbacks"]["dirpath"],
        filename="best_model",
        monitor=training_params["callbacks"]["monitor_var"],
        mode=training_params["callbacks"]["monitor_var_mode"],
        save_top_k=3,  # Save the top 3 models based on the monitored metric
    )
    callbacks = [
        LearningRateMonitor(logging_interval="step"),
        model_checkpoint,
    ]

    return callbacks


def load_model(
    training_params: Dict,
) -> Tuple[Dict, AutoModelForSequenceClassification]:
    # load HF configs
    model_config = AutoConfig.from_pretrained(training_params.get("base-model-name"))

    # Load model (downloads from hub for the first time)
    model = Model.from_config(training_params.get("train_mode"))
    model = model.from_pretrained(training_params.get("base-model-name"), config=model_config)

    # Attach custom classifier
    model.classifier = CustomClassificationHead(**training_params.get("custom_classification_head"))

    for params in model.parameters():
        params.requires_grad = False

    for params in model.classifier.parameters():
        params.requires_grad = True
    return model_config, model


def get_dataset(
    dataset_params: Dict, tokenizer: AutoTokenizer
) -> Tuple[SentimentIterableDataset, SentimentIterableDataset]:
    dataset_obj = InitializeDataset(dataset_params)
    raw_dataset_file, _ = dataset_obj()
    # Load streaming dataset
    train_dataset = SentimentIterableDataset(
        raw_dataset_file,
        tokenizer,
        split_type="train",
        preprocessors=dataset_params.get("preprocessors"),
    )
    valid_dataset = SentimentIterableDataset(
        raw_dataset_file,
        tokenizer,
        split_type="valid",
        preprocessors=dataset_params.get("preprocessors"),
    )
    return train_dataset, valid_dataset


def get_trainer(
    loggers: List[WandbLogger],
    devices: List[str] | str,
    accelerator: str,
    callbacks: List,
    training_params: Dict,
) -> pl.Trainer:
    trainer = pl.Trainer(
        logger=loggers,
        devices=devices,
        accelerator=accelerator,
        callbacks=callbacks,
        **training_params["trainer"],
    )
    return trainer


def train(
    config: dict,
    device: list[str],
    training_params: Dict,
    dataset_params: Dict,
    seed: int,
    inference_params: Dict,
):
    # wandb login
    wandb.login(key=os.getenv("WANDB_API_KEY"))

    # Global seeding
    pl.seed_everything(seed=seed)
    model_config, model = load_model(training_params)
    tokenizer = AutoTokenizer.from_pretrained(training_params.get("base-model-name"))
    train_dataset, valid_dataset = get_dataset(dataset_params, tokenizer)
    loggers = init_model_loggers(training_params)
    callbacks = init_model_callbacks(training_params)

    # PL wrapper
    model_wrapped = LightningModelWrapper(
        model=model,
        optimizer_params=training_params["optimizer"],
        lr_scheduler_params=(training_params["lr_scheduler"] if "lr_scheduler" in training_params else None),
        unique_config=config,
    ).to(device)

    # Get available devices
    if "cuda" in device:
        devices = [int(device.split(":")[-1])]
        accelerator = "gpu"
    else:
        devices = "auto"
        accelerator = "cpu"

    # Trainer initialization
    trainer = get_trainer(loggers, devices, accelerator, callbacks, training_params)

    # Initiate trainer
    trainer.fit(
        model=model_wrapped,
        train_dataloaders=train_dataset,
        val_dataloaders=valid_dataset,
        ckpt_path="last",
    )

    with wandb.init(project=training_params["wandb_storage"]["name"]) as run:
        best_model = wandb.Artifact(
            f"{training_params['wandb_storage']['artifact_name']}",
            type="model",
        )
        best_model.add_file(callbacks[1].best_model_path)
        run.log_artifact(best_model)

    wandb.finish()


def main():
    args = parse_args()
    config = yaml.safe_load(open(args.config, "r"))
    device = "cuda" if torch.cuda.is_available() else "cpu"
    if device == "cuda":
        device = "cuda:3"  # Hardcoded device id
    if args.mode == "train":
        train(config=config, device=device, **config)
    else:
        raise NotImplementedError(f"Method {args.mode} not implemented")


if __name__ == "__main__":
    main()



import os.path
from glob import glob
from typing import Dict, Generator, List, Optional, Tuple

import pandas as pd
import torch
import wandb
from sa_app.data.data_cleaner import StackedPreprocessor
from sa_app.data.kaggle_dataset import get_dataset_length, get_file_names, split_dataset
from torch.utils.data import IterableDataset
from tqdm import tqdm
from transformers import AutoTokenizer


def kaggle_dataset_iterator(file_map: dict, chunk_size=1000, split_type="train") -> pd.DataFrame:
    dataset_path = file_map[split_type]
    return pd.read_csv(dataset_path, encoding="latin-1", chunksize=chunk_size)


class InitializeDataset:
    def __init__(self, dataset_params):
        self.dataset_params = dataset_params
        self.wandb_storage = self.dataset_params.get("wandb_storage")
        self.local_storage = self.dataset_params.get("local_storage")

    def __call__(self, *args, **kwargs) -> Tuple[str, str]:

        if self.wandb_storage is not None:
            storage_url = self.wandb_storage.get("wandb_dataset_url")
            wandb_file_type = self.wandb_storage.get("src_file_type")
            mapping_file_name = self.wandb_storage.get("mapping_file_name")

            # Download artifacts from wandb
            run = wandb.init()
            artifact = run.use_artifact(storage_url, type="training_dataset")
            artifact_dir = artifact.download()

            # Artifact Validation
            assert len(glob(f"{artifact_dir}/*.{wandb_file_type}")) > 0, "CSV file download failed"
            csv_file = glob(f"{artifact_dir}/*.{wandb_file_type}")[0]
            mapping_file = os.path.join(artifact_dir, mapping_file_name)
            assert os.path.isfile(mapping_file) is True, "Label mapping file download failed"

            return csv_file, mapping_file

        elif self.local_storage is not None:
            local_storage = self.local_storage
            csv_file = local_storage.get("raw_dataset_file")
            mapping_file = local_storage.get("labels_mapping")
            return csv_file, mapping_file
        else:
            raise NotImplementedError("Either of wandb_storage or local_storage should be defined in app_cfg.yml")


class SentimentIterableDataset(IterableDataset):
    def __init__(
        self,
        csv_file: str,
        tokenizer,
        preprocessors: Optional[Dict] = None,
        chunk_size: int = 1000,
        create_split: bool = False,
        split_type: str = "train",
        batch_size: int = 8,
        max_seq_len: int = 512,
    ):
        self.csv_file = csv_file
        self.tokenizer = tokenizer
        self.chunk_size = chunk_size
        self.create_split = create_split
        self.split_type = split_type
        self.max_seq_len = max_seq_len
        self.batch_size = batch_size
        self.preprocessors = StackedPreprocessor(preprocessors)
        data_files = get_file_names(self.csv_file)
        self.file_map = {
            "train": data_files[0],
            "valid": data_files[1],
            "test": data_files[2],
        }
        if os.path.isfile(self.file_map[split_type]) is False:
            print("Splitting the dataset")
            split_dataset(self.csv_file)

    def __len__(self):
        return get_dataset_length(self.csv_file, self.split_type) // self.batch_size

    def __iter__(self) -> Generator[Tuple[List[str], Dict[str, List[str]]], None, None]:
        for data in kaggle_dataset_iterator(self.file_map, chunk_size=self.chunk_size, split_type=self.split_type):
            for i in range(0, len(data), self.batch_size):
                # Label mapping is also done here, 0 - negative sentiment, 1 - positive sentiment
                labels_minibatch: List[int] = list(
                    data.iloc[i : i + self.batch_size, 0].apply(lambda x: 0 if x == 0 else 1).values
                )

                sentences_minibatch: Dict[str, List[str]] = self.tokenizer(
                    self.preprocessors(list(data.iloc[i : i + self.batch_size, 5].values)),
                    add_special_tokens=True,
                    truncation=True,
                    max_length=self.max_seq_len,
                    padding=True,
                )

                labels_tensor = torch.tensor(labels_minibatch)

                sentences = {
                    "input_ids": torch.tensor(sentences_minibatch["input_ids"]),
                    "attention_mask": torch.tensor(sentences_minibatch["attention_mask"]),
                }
                yield labels_tensor, sentences


if __name__ == "__main__":
    tokenizer = AutoTokenizer.from_pretrained("cardiffnlp/twitter-roberta-base")
    raw_dataset_file = (
        "/home/ppradhan/Documents/my_learnings/my_uni_stuffs/sa_data_storage/training.1600000"
        ".processed.noemoticon.csv"
    )
    # ite = kaggle_dataset_iterator(raw_dataset_file, chunk_size=1000, create_split=False, split_type='train')
    dataset = SentimentIterableDataset(raw_dataset_file, tokenizer)

    total_dataset_length = len(dataset)
    pbar = tqdm(total=total_dataset_length, desc="Processing batches", unit="batch")

    for batch in dataset:
        labels, sentences = batch
        pbar.update(1)



import inspect
from importlib import import_module

import torch

OPTIMIZERS = {
    "adam": "torch.optim.Adam",
    "adamW": "torch.optim.AdamW",
    "adagrad": "torch.optim.Adagrad",
}

LR_SCHEDULERS = {
    "exponential": "torch.optim.lr_scheduler.ExponentialLR",
    "lin_warmup": "transformers.get_linear_schedule_with_warmup",
}


class Optimizer:
    @classmethod
    def from_config(cls, type_: str, *args, **kwargs) -> torch.optim.Optimizer:
        try:
            callable_path = OPTIMIZERS[type_]
        except KeyError:
            raise KeyError(f'Optimizer "{type_}" is not implemented.')

        module_name, class_name = callable_path.rsplit(".", maxsplit=1)
        module = import_module(module_name)
        class_ = getattr(module, class_name)
        return class_(*args, **kwargs)


class LearningRateScheduler:
    @classmethod
    def from_config(cls, type_: str, optimizer: torch.optim.Optimizer, **kwargs):
        try:
            callable_path = LR_SCHEDULERS[type_]
        except KeyError:
            raise KeyError(f'{cls.__name__} "{type_}" is not implemented.')

        module_name, class_name = callable_path.rsplit(".", maxsplit=1)
        module = import_module(module_name)
        class_ = getattr(module, class_name)

        expected_scheduler_args = inspect.signature(class_).parameters.keys()
        scheduler_kwargs = {name: value for name, value in kwargs.items() if name in expected_scheduler_args}

        return class_(optimizer=optimizer, **scheduler_kwargs)



from typing import Dict

import pandas as pd


def get_label_counts(df: pd.DataFrame) -> Dict[str, int]:
    label_count = {}
    for i in df:
        k, v = list(i[0].value_counts().to_dict().items())[0]
        if k not in label_count:
            label_count[k] = 0
        label_count[k] += v
    return label_count


def get_dataset_length(file_name: str, split_type: str) -> int:
    data_files = get_file_names(file_name)
    file_map = {"train": data_files[0], "valid": data_files[1], "test": data_files[2]}
    return sum([len(i) for i in pd.read_csv(file_map[split_type], encoding="latin-1", chunksize=1000)])


def get_file_names(file_name: str) -> tuple[str, str, str]:
    file_base_pth, file_ext = file_name.rsplit(".", 1)
    train_file = f"{file_base_pth}.train.{file_ext}"
    valid_file = f"{file_base_pth}.valid.{file_ext}"
    test_file = f"{file_base_pth}.test.{file_ext}"
    return train_file, valid_file, test_file


def split_dataset(
    file: str,
    train_ratio: float = 0.7,
    test_ratio: float = 0.15,
    valid_ratio: float = 0.15,
) -> bool:
    train_file, valid_file, test_file = get_file_names(file)

    # Load the entire dataset
    df = pd.read_csv(file, encoding="latin-1", header=None)

    # Shuffle the dataset
    df = df.sample(frac=1, random_state=42).reset_index(drop=True)

    # Calculate the split indices
    total_samples = len(df)
    train_idx = int(train_ratio * total_samples)
    valid_idx = int(valid_ratio * total_samples)

    # Split the dataset
    train_df = df.iloc[:train_idx]
    valid_df = df.iloc[train_idx : train_idx + valid_idx]
    test_df = df.iloc[train_idx + valid_idx :]

    # Save the datasets
    train_df.to_csv(train_file, mode="w", header=False, index=False)
    valid_df.to_csv(valid_file, mode="w", header=False, index=False)
    test_df.to_csv(test_file, mode="w", header=False, index=False)

    print("Data split completed successfully!")
    return True


if __name__ == "__main__":
    raw_dataset_file = (
        "/home/ppradhan/Documents/my_learnings/my_uni_stuffs/sa_data_storage/training.1600000"
        ".processed.noemoticon.csv"
    )
    print(split_dataset(raw_dataset_file))



import os
from typing import Dict

import torch
import wandb
import yaml
from sa_app.common.utils import parse_args
from sa_app.data.data_cleaner import StackedPreprocessor
from transformers import AutoConfig, AutoModelForSequenceClassification, AutoTokenizer


class InferenceEngine:
    def __init__(
        self,
        inference_params: Dict,
        training_params: Dict,
        dataset_params: Dict,
        device: str,
    ):
        # Set the device to CPU or GPU
        self.device = device

        # Load the trained model and tokenizer
        self.model, self.model_config = self.get_wandb_model(inference_params)
        self.tokenizer = AutoTokenizer.from_pretrained(training_params["tokenizer"])

        # Preprecessor
        self.preprocessors = StackedPreprocessor(dataset_params["preprocessors"])

        # Dataset initialization
        # dataset_obj = InitializeDataset(dataset_params)
        # _, label_mapping_path = dataset_obj()
        # self.label_mapping = load_mapping(label_mapping_path)

    def get_wandb_model(self, inference_params):
        # Download checkpoint from wandb
        run = wandb.init()
        artifact = run.use_artifact(inference_params["model_dir"], type="model")
        artifact_dir = artifact.download()

        # Load checkpoint into the pretrained model
        checkpoint_pth = os.path.join(artifact_dir, inference_params["default_model_name"])
        state_dict = torch.load(checkpoint_pth)
        new_state_dict = {}
        for key in state_dict["state_dict"].keys():
            if key.startswith("model."):
                new_key = key.replace("model.", "")  # Remove 'model.' prefix
                new_state_dict[new_key] = state_dict["state_dict"][key]

        config = AutoConfig.from_pretrained(inference_params["base_model_name"])
        model = AutoModelForSequenceClassification.from_config(config).to(self.device)
        # model.load_state_dict(new_state_dict)

        return model, config

    def perform_inference(self, sentence):
        # Preprocess the input sentence
        sentence = self.preprocessors([sentence])[0]

        # Tokenize the input sentence
        inputs = self.tokenizer(sentence, truncation=True, padding=True, return_tensors="pt")
        inputs = inputs.to(self.device)

        # Eval mode
        self.model.eval()

        # Forward pass through the model
        with torch.no_grad():
            outputs = self.model(**inputs)

        # Get the predicted labels
        probs = torch.softmax(outputs[0][0], dim=-1)
        predicted_labels = torch.argmax(probs, dim=-1)

        return self.model_config.id2label[predicted_labels.tolist()]


if __name__ == "__main__":
    args = parse_args()
    config = yaml.safe_load(open(args.config, "r"))
    device_in_use = "cuda" if torch.cuda.is_available() else "cpu"
    # Example usage
    ie_obj = InferenceEngine(
        inference_params=config["inference_params"],
        training_params=config["training_params"],
        dataset_params=config["dataset_params"],
        device=device_in_use,
    )
    # input_sentence = "I feel so bad today . Such a bad day :( "
    # input_sentence = """
    # @swikey haha, okay, i feel much better now.
    # let's just dye our hair paramore red!
    # """
    input_sentence = "Covid cases are increasing fast!"
    predicted_labels = ie_obj.perform_inference(input_sentence)
    print("Predicted labels:", predicted_labels)



from enum import Enum
from pathlib import Path
from typing import Any, Dict, Optional, Union

import pytorch_lightning as pl
import torch
from sa_app.training.optmizer import LearningRateScheduler, Optimizer
from torch.nn import CrossEntropyLoss
from torchmetrics import Accuracy, MeanMetric, Metric
from transformers import AutoModelForSequenceClassification


class Split(str, Enum):
    TRAIN = "train"
    VALID = "valid"
    TEST = "test"

    def __str__(self):
        return self.value


class LightningModelWrapper(pl.LightningModule):
    def __init__(
        self,
        model: AutoModelForSequenceClassification,
        optimizer_params: Optional[dict] = None,
        lr_scheduler_params: Optional[dict] = None,
        unique_config: Optional[dict] = None,
    ):
        super().__init__()

        self.model = model

        # Optional: For Training only
        self.optimizer_params = optimizer_params
        self.lr_scheduler_params = lr_scheduler_params
        self.unique_config = unique_config
        self.num_classes = unique_config.get("training_params").get("custom_classification_head").get("num_labels")
        self.task_name = "multiclass" if self.num_classes > 2 else "binary"
        self.loss_fn = CrossEntropyLoss()
        self.softmax = torch.nn.Softmax(dim=-1)

        self.train_loss = MeanMetric()
        self.train_acc = Accuracy(task=self.task_name, num_classes=self.num_classes)

        self.valid_loss = MeanMetric()
        self.valid_acc = Accuracy(task=self.task_name, num_classes=self.num_classes)

    def forward(self, x):
        labels, sentence_batch = x
        [sentence_batch[inp_key].to(self.model.device) for inp_key, value in sentence_batch.items()]
        output = self.model(**sentence_batch)

        probs = self.softmax(output.logits[:, 0, :])
        # predicted_classes = torch.argmax(probs, dim=-1)

        output.loss = self.loss_fn(output.logits[:, 0, :], labels)
        return {"logits": probs, "loss": output.loss}

    def training_step(self, batch: dict, batch_idx: int) -> Dict:
        return self.forward(batch)

    def on_train_batch_end(self, step_output: Dict, batch: Any, batch_idx: int) -> None:
        self.log_batch_end(
            step_output=step_output,
            batch=batch,
            loss_fn=self.train_loss,
            acc_fn=self.train_acc,
            split_type=Split.TRAIN,
        )

    def validation_step(self, batch: dict, batch_idx: int) -> Dict:
        return self.forward(batch)

    def on_validation_batch_end(self, step_output: Dict, batch: Any, batch_idx: int, dataloader_idx: int = 0) -> None:
        self.log_batch_end(
            step_output=step_output,
            batch=batch,
            loss_fn=self.valid_loss,
            acc_fn=self.valid_acc,
            split_type=Split.VALID,
        )

    def on_train_epoch_end(self) -> None:
        self.log_epoch_end(loss_fn=self.train_loss, split_type=Split.TRAIN)

    def on_validation_epoch_end(self) -> None:
        self.log_epoch_end(loss_fn=self.valid_loss, split_type=Split.VALID)

    def log_batch_end(
        self,
        step_output: dict,
        batch: dict,
        loss_fn: Metric,
        acc_fn: Metric,
        split_type: Split,
    ):
        loss = step_output.get("loss")
        loss_fn.update(loss)
        self.log(
            f"{split_type}_loss_step",
            loss,
            batch_size=self.trainer.train_dataloader.batch_size,
        )

        labels, sentence_batch = batch
        predicted_labels = torch.argmax(step_output.get("logits"), dim=1)
        self.log(
            f"{split_type}_acc_step",
            acc_fn(predicted_labels, labels),
            batch_size=self.trainer.train_dataloader.batch_size,
        )

    def log_epoch_end(self, loss_fn: Metric, split_type: Split):
        loss_fn = loss_fn.compute() if loss_fn.mean_value != 0 else None
        if loss_fn is not None:
            self.log(f"{split_type}-loss-epoch", loss_fn)

    def configure_optimizers(self):
        optimizer = Optimizer.from_config(params=self.parameters(), **self.optimizer_params)

        scheduler = None
        if self.lr_scheduler_params is not None:
            self.trainer.fit_loop.setup_data()
            train_steps = self.trainer.max_steps
            lr_warmup = self.lr_scheduler_params.pop("lr_warmup", 0.0)
            interval = self.lr_scheduler_params.pop("interval", "epoch")
            lr_scheduler = LearningRateScheduler.from_config(
                optimizer=optimizer,
                num_warmup_steps=lr_warmup * train_steps,
                num_training_steps=train_steps,
                **self.lr_scheduler_params,
            )

            scheduler = {
                "scheduler": lr_scheduler,
                "interval": interval,
                "frequency": 1,
                "strict": False,
                "monitor": "loss",
            }

        if scheduler:
            return [optimizer], [scheduler]
        else:
            return optimizer

    def save_model(self, path: Union[str, Path]) -> None:
        """Save the model using the original HF AutoModel.

        This is useful for when you'd like to export the model to the hub.
        Args:
            path: Path to save the model to.
        """
        self.model.save_pretrained(path)
